#include "fd_event.hpp"
#include "logger.hpp"
#include <cstring>
#include <functional>
#include <unistd.h>
#include <fcntl.h>
namespace tinyrpc {
    FdEvent::FdEvent(int fd) : m_fd(fd) {

    }

    FdEvent::FdEvent(int fd, Reactor* reactor) : m_fd(fd), m_reactor(reactor) {

    }

    bool FdEvent::setNonblock() {
        if (m_fd < 0) {
            logger() << "error fd < 0 !";
            return false;
        }

        int flag = fcntl(m_fd, F_GETFL);

        if(flag == -1) {
            logger() << "fcntl return -1 err:" << strerror(errno);
            return false;
        }

        flag |= O_NONBLOCK;

        int ret = fcntl(m_fd, F_SETFL, flag);

        if(ret == -1) {
            logger() << "fcntl return -1 err:" << strerror(errno);
            return false;
        }

        return true;
    }
    std::function<void()> FdEvent::getHandler(IOEvent event) const {
        if(event == IOEvent::READ) return m_read_callback;
        else if (event == IOEvent::WRITE)  return m_write_callback;
        logger() << "unknow IOEvent!";
        return nullptr;
    }

    void FdEvent::addListenEvent(IOEvent event) {
        int ev = static_cast<int>(event);
        if(m_listen_events & ev) {
            logger() << "already has this event";
            return;
        }
        m_listen_events |= static_cast<int>(event);
    }
    void FdEvent::delListenEvent(IOEvent event) {
        int ev = static_cast<int>(event);
        if(!(m_listen_events & ev)) {
            logger() << "this event not exist";
            return;
        }
        m_listen_events &= ~static_cast<int>(event);
    }
}